@@ -33,6 +33,7 @@
 #include "icons.h"
 #include "pretty-print.h"
 #include "gui.h"
+#include "server-device.h"
 
 #define CONTENT_DIR "urn:schemas-upnp-org:service:ContentDirectory"
 
@@ -74,13 +75,16 @@ void
 on_didl_menuitem_activate (GtkMenuItem *menuitem,
                            gpointer     user_data);
 
+static void
+on_proxy_ready (GObject *source_object,
+                GAsyncResult *res,
+                gpointer user_data);
+
 typedef struct
 {
-  GUPnPServiceProxy *content_dir;
-
-  gchar *id;
-
-  guint32 starting_index;
+        AVCPMediaServer *server;
+        gchar *id;
+        guint32 starting_index;
 } BrowseData;
 
 typedef struct
@@ -93,14 +97,14 @@ typedef struct
 } BrowseMetadataData;
 
 static BrowseData *
-browse_data_new (GUPnPServiceProxy *content_dir,
-                 const char        *id,
-                 guint32            starting_index)
+browse_data_new (AVCPMediaServer *server,
+                 const char      *id,
+                 guint32          starting_index)
 {
         BrowseData *data;
 
         data = g_slice_new (BrowseData);
-        data->content_dir = g_object_ref (content_dir);
+        data->server = g_object_ref (server);
         data->id = g_strdup (id);
         data->starting_index = starting_index;
 
@@ -111,7 +115,7 @@ static void
 browse_data_free (BrowseData *data)
 {
         g_free (data->id);
-        g_object_unref (data->content_dir);
+        g_object_unref (data->server);
         g_slice_free (BrowseData, data);
 }
 
@@ -138,10 +142,10 @@ browse_metadata_data_free (BrowseMetadataData *data)
 }
 
 static void
-browse (GUPnPServiceProxy *content_dir,
-        const char        *container_id,
-        guint32            starting_index,
-        guint32            requested_count);
+browse (AVCPMediaServer *content_dir,
+        const char      *container_id,
+        guint32          starting_index,
+        guint32          requested_count);
 
 G_MODULE_EXPORT
 gboolean
@@ -270,23 +274,23 @@ on_playlist_row_expanded (GtkTreeView *tree_view,
         }
 
         do {
-                GUPnPServiceProxy *content_dir;
-                gchar             *id;
-                gboolean           is_container;
-                gint              child_count;
+                AVCPMediaServer *server;
+                gchar           *id;
+                gboolean         is_container;
+                gint             child_count;
 
                 gtk_tree_model_get (model, &child_iter,
-                                    3, &content_dir,
+                                    2, &server,
                                     4, &id,
                                     5, &is_container,
                                     6, &child_count,
                                     -1);
 
                 if (is_container && child_count != 0) {
-                        browse (content_dir, id, 0, MAX_BROWSE);
+                        browse (server, id, 0, MAX_BROWSE);
                 }
 
-                g_object_unref (content_dir);
+                g_object_unref (server);
                 g_free (id);
         } while (gtk_tree_model_iter_next (model, &child_iter));
 }
@@ -561,7 +565,7 @@ get_item_icon (GUPnPDIDLLiteObject *object)
 }
 
 static gboolean
-find_container (GUPnPServiceProxy *content_dir,
+find_container (AVCPMediaServer   *server,
                 GtkTreeModel      *model,
                 GtkTreeIter       *container_iter,
                 const char        *container_id)
@@ -569,7 +573,7 @@ find_container (GUPnPServiceProxy *content_dir,
         GtkTreeIter server_iter;
         const char *udn;
 
-        udn = gupnp_service_info_get_udn (GUPNP_SERVICE_INFO (content_dir));
+        udn = gupnp_device_info_get_udn (GUPNP_DEVICE_INFO (server));
 
         if (!find_row (model,
                        NULL,
@@ -592,8 +596,8 @@ find_container (GUPnPServiceProxy *content_dir,
 }
 
 static void
-update_container (GUPnPServiceProxy *content_dir,
-                  const char        *container_id)
+update_container (AVCPMediaServer *server,
+                  const char      *container_id)
 {
         GtkTreeModel *model;
         GtkTreeIter   container_iter;
@@ -601,7 +605,7 @@ update_container (GUPnPServiceProxy *content_dir,
         model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
         g_assert (model != NULL);
 
-        if (find_container (content_dir,
+        if (find_container (server,
                             model,
                             &container_iter,
                             container_id)) {
@@ -611,13 +615,13 @@ update_container (GUPnPServiceProxy *content_dir,
                 }
 
                 /* Browse it again */
-                browse (content_dir, container_id, 0, MAX_BROWSE);
+                browse (server, container_id, 0, MAX_BROWSE);
         }
 }
 
 static void
-update_container_child_count (GUPnPServiceProxy *content_dir,
-                              const char        *container_id)
+update_container_child_count (AVCPMediaServer *server,
+                              const char      *container_id)
 {
         GtkTreeModel *model;
         GtkTreeIter   container_iter;
@@ -625,7 +629,7 @@ update_container_child_count (GUPnPServiceProxy *content_dir,
         model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
         g_assert (model != NULL);
 
-        if (find_container (content_dir,
+        if (find_container (server,
                             model,
                             &container_iter,
                             container_id)) {
@@ -648,6 +652,7 @@ on_container_update_ids (GUPnPServiceProxy *content_dir,
 {
         char **tokens;
         guint  i;
+        AVCPMediaServer *server = AV_CP_MEDIA_SERVER (user_data);
 
         /* Ignore initial event. It might have some old updates that happened
          * ages ago and cause duplicate entries in our list.
@@ -662,7 +667,7 @@ on_container_update_ids (GUPnPServiceProxy *content_dir,
         for (i = 0;
              tokens[i] != NULL && tokens[i+1] != NULL;
              i += 2) {
-                update_container (content_dir, tokens[i]);
+                update_container (server, tokens[i]);
         }
 
         g_strfreev (tokens);
@@ -681,7 +686,6 @@ append_didl_object (GUPnPDIDLLiteObject *object,
         gboolean    is_container;
         gint        child_count;
         GdkPixbuf  *icon;
-        gint        position;
 
         id = gupnp_didl_lite_object_get_id (object);
         title = gupnp_didl_lite_object_get_title (object);
@@ -693,14 +697,12 @@ append_didl_object (GUPnPDIDLLiteObject *object,
 
         if (is_container) {
                 GUPnPDIDLLiteContainer *container;
-                position = 0;
                 icon = get_icon_by_id (ICON_CONTAINER);
 
                 container = GUPNP_DIDL_LITE_CONTAINER (object);
                 child_count = gupnp_didl_lite_container_get_child_count
                                                                 (container);
         } else {
-                position = -1;
                 child_count = 0;
                 icon = get_item_icon (object);
         }
@@ -717,10 +719,10 @@ append_didl_object (GUPnPDIDLLiteObject *object,
                 return;
 
         gtk_tree_store_insert_with_values (GTK_TREE_STORE (model),
-                                           NULL, &parent_iter, position,
+                                           NULL, &parent_iter, -1,
                                            0, icon,
                                            1, title,
-                                           3, browse_data->content_dir,
+                                           2, browse_data->server,
                                            4, id,
                                            5, is_container,
                                            6, child_count,
@@ -728,17 +730,20 @@ append_didl_object (GUPnPDIDLLiteObject *object,
 }
 
 static void
-on_device_icon_available (GUPnPDeviceInfo *info,
-                          GdkPixbuf       *icon)
+update_device_icon (GUPnPDeviceInfo *info)
 {
         GtkTreeModel *model;
         GtkTreeIter   iter;
+        AVCPMediaServer *server;
         const char   *udn;
+        GdkPixbuf    *icon;
 
         model = gtk_tree_view_get_model (GTK_TREE_VIEW (treeview));
         g_assert (model != NULL);
 
         udn = gupnp_device_info_get_udn (info);
+        server = AV_CP_MEDIA_SERVER (info);
+        g_object_get (G_OBJECT (server), "icon", &icon, NULL);
 
         if (find_row (model,
                       NULL,
@@ -755,19 +760,25 @@ on_device_icon_available (GUPnPDeviceInfo *info,
         g_object_unref (icon);
 }
 
-static GUPnPServiceProxy *
-get_content_dir (GUPnPDeviceProxy *proxy)
+static void
+on_proxy_ready (GObject *source_object,
+                GAsyncResult *res,
+                gpointer user_data)
 {
-        GUPnPDeviceInfo  *info;
-        GUPnPServiceInfo *content_dir;
+        GError *error = NULL;
+        gboolean result = FALSE;
 
-        info = GUPNP_DEVICE_INFO (proxy);
+        result = g_async_initable_init_finish (G_ASYNC_INITABLE (source_object),
+                                               res,
+                                               &error);
 
-        content_dir = gupnp_device_info_get_service (info, CONTENT_DIR);
-
-        return GUPNP_SERVICE_PROXY (content_dir);
+        if (result == TRUE) {
+                update_device_icon (GUPNP_DEVICE_INFO (source_object));
+                browse (AV_CP_MEDIA_SERVER (source_object), "0", 0, MAX_BROWSE);
+        }
 }
 
+
 static void
 on_didl_object_available (GUPnPDIDLLiteParser *parser,
                           GUPnPDIDLLiteObject *object,
@@ -783,8 +794,8 @@ on_didl_object_available (GUPnPDIDLLiteParser *parser,
 
         browse_data = (BrowseData *) user_data;
 
-        udn = gupnp_service_info_get_udn
-                                (GUPNP_SERVICE_INFO (browse_data->content_dir));
+        udn = gupnp_device_info_get_udn
+                                (GUPNP_DEVICE_INFO (browse_data->server));
 
         if (find_row (model,
                        NULL,
@@ -802,9 +813,9 @@ on_didl_object_available (GUPnPDIDLLiteParser *parser,
 }
 
 static void
-browse_cb (GUPnPServiceProxy       *content_dir,
-           GUPnPServiceProxyAction *action,
-           gpointer                 user_data)
+browse_cb (GObject *object,
+           GAsyncResult *result,
+           gpointer user_data)
 {
         BrowseData *data;
         char       *didl_xml;
@@ -816,20 +827,13 @@ browse_cb (GUPnPServiceProxy       *content_dir,
         didl_xml = NULL;
         error = NULL;
 
-        gupnp_service_proxy_end_action (content_dir,
-                                        action,
-                                        &error,
-                                        /* OUT args */
-                                        "Result",
-                                        G_TYPE_STRING,
-                                        &didl_xml,
-                                        "NumberReturned",
-                                        G_TYPE_UINT,
-                                        &number_returned,
-                                        "TotalMatches",
-                                        G_TYPE_UINT,
-                                        &total_matches,
-                                        NULL);
+        av_cp_media_server_browse_finish (AV_CP_MEDIA_SERVER (object),
+                                          result,
+                                          &didl_xml,
+                                          &total_matches,
+                                          &number_returned,
+                                          &error);
+
         if (didl_xml) {
                 GUPnPDIDLLiteParser *parser;
                 gint32              remaining;
@@ -870,18 +874,16 @@ browse_cb (GUPnPServiceProxy       *content_dir,
                         else
                                 batch_size = MAX_BROWSE;
 
-                        browse (content_dir,
+                        browse (AV_CP_MEDIA_SERVER (object),
                                 data->id,
                                 data->starting_index,
                                 batch_size);
                 } else
-                        update_container_child_count (content_dir, data->id);
+                        update_container_child_count (AV_CP_MEDIA_SERVER (object),
+                                                      data->id);
         } else if (error) {
-                GUPnPServiceInfo *info;
-
-                info = GUPNP_SERVICE_INFO (content_dir);
                 g_warning ("Failed to browse '%s': %s",
-                           gupnp_service_info_get_location (info),
+                           gupnp_device_info_get_location (GUPNP_DEVICE_INFO (object)),
                            error->message);
 
                 g_error_free (error);
@@ -891,9 +893,9 @@ browse_cb (GUPnPServiceProxy       *content_dir,
 }
 
 static void
-browse_metadata_cb (GUPnPServiceProxy       *content_dir,
-                    GUPnPServiceProxyAction *action,
-                    gpointer                 user_data)
+browse_metadata_cb (GObject      *object,
+                    GAsyncResult *result,
+                    gpointer      user_data)
 {
         BrowseMetadataData *data;
         char               *metadata;
@@ -904,15 +906,12 @@ browse_metadata_cb (GUPnPServiceProxy       *content_dir,
         metadata = NULL;
         error = NULL;
 
-        gupnp_service_proxy_end_action (content_dir,
-                                        action,
-                                        &error,
-                                        /* OUT args */
-                                        "Result",
-                                        G_TYPE_STRING,
-                                        &metadata,
-                                        NULL);
-        if (metadata) {
+        av_cp_media_server_browse_metadata_finish (AV_CP_MEDIA_SERVER (object),
+                                                   result,
+                                                   &metadata,
+                                                   &error);
+
+        if (metadata != NULL) {
                 data->callback (metadata, data->user_data);
 
                 g_free (metadata);
@@ -925,82 +924,44 @@ browse_metadata_cb (GUPnPServiceProxy       *content_dir,
         }
 
         browse_metadata_data_free (data);
-        g_object_unref (content_dir);
 }
 
 static void
-browse (GUPnPServiceProxy *content_dir,
-        const char        *container_id,
-        guint32            starting_index,
-        guint32            requested_count)
+browse (AVCPMediaServer *server,
+        const char      *container_id,
+        guint32          starting_index,
+        guint32          requested_count)
 {
         BrowseData *data;
 
-        data = browse_data_new (content_dir,
+        data = browse_data_new (server,
                                 container_id,
                                 starting_index);
 
-        gupnp_service_proxy_begin_action
-                                (content_dir,
-                                 "Browse",
-                                 browse_cb,
-                                 data,
-                                 /* IN args */
-                                 "ObjectID",
-                                 G_TYPE_STRING,
-                                 container_id,
-                                 "BrowseFlag",
-                                 G_TYPE_STRING,
-                                 "BrowseDirectChildren",
-                                 "Filter",
-                                 G_TYPE_STRING,
-                                 "@childCount",
-                                 "StartingIndex",
-                                 G_TYPE_UINT,
-                                 starting_index,
-                                 "RequestedCount",
-                                 G_TYPE_UINT,
-                                 requested_count,
-                                 "SortCriteria",
-                                 G_TYPE_STRING,
-                                 "",
-                                 NULL);
+        av_cp_media_server_browse_async (server,
+                                         NULL,
+                                         browse_cb,
+                                         container_id,
+                                         starting_index,
+                                         requested_count,
+                                         data);
 }
 
 static void
-browse_metadata (GUPnPServiceProxy *content_dir,
-                 const char        *id,
-                 MetadataFunc       callback,
-                 gpointer           user_data)
+browse_metadata (AVCPMediaServer *server,
+                 const char      *id,
+                 MetadataFunc     callback,
+                 gpointer         user_data)
 {
         BrowseMetadataData *data;
 
         data = browse_metadata_data_new (callback, id, user_data);
 
-        gupnp_service_proxy_begin_action
-                                (g_object_ref (content_dir),
-                                 "Browse",
-                                 browse_metadata_cb,
-                                 data,
-                                 /* IN args */
-                                 "ObjectID",
-                                 G_TYPE_STRING,
-                                 data->id,
-                                 "BrowseFlag",
-                                 G_TYPE_STRING,
-                                 "BrowseMetadata",
-                                 "Filter",
-                                 G_TYPE_STRING,
-                                 "*",
-                                 "StartingIndex",
-                                 G_TYPE_UINT,
-                                 0,
-                                 "RequestedCount",
-                                 G_TYPE_UINT, 0,
-                                 "SortCriteria",
-                                 G_TYPE_STRING,
-                                 "",
-                                 NULL);
+        av_cp_media_server_browse_metadata_async (server,
+                                                  NULL,
+                                                  browse_metadata_cb,
+                                                  id,
+                                                  data);
 }
 
 static void
@@ -1009,13 +970,14 @@ append_media_server (GUPnPDeviceProxy *proxy,
                      GtkTreeIter      *parent_iter)
 {
         GUPnPDeviceInfo   *info;
-        GUPnPServiceProxy *content_dir;
         char              *friendly_name;
+        GUPnPServiceProxy *content_dir;
 
         info = GUPNP_DEVICE_INFO (proxy);
 
-        content_dir = get_content_dir (proxy);
         friendly_name = gupnp_device_info_get_friendly_name (info);
+        content_dir = av_cp_media_server_get_content_directory
+                                (AV_CP_MEDIA_SERVER (info));
 
         if (G_LIKELY (friendly_name != NULL && content_dir != NULL)) {
                 GtkTreeIter device_iter;
@@ -1033,6 +995,12 @@ append_media_server (GUPnPDeviceProxy *proxy,
                                  -1);
                 g_free (friendly_name);
 
+                g_async_initable_init_async (G_ASYNC_INITABLE (proxy),
+                                             G_PRIORITY_DEFAULT,
+                                             NULL,
+                                             on_proxy_ready,
+                                             content_dir);
+
                 /* Append the embedded devices */
                 child = gupnp_device_info_list_devices (info);
                 while (child) {
@@ -1043,9 +1011,6 @@ append_media_server (GUPnPDeviceProxy *proxy,
                         child = g_list_delete_link (child, child);
                 }
 
-                schedule_icon_update (info, on_device_icon_available);
-
-                browse (content_dir, "0", 0, MAX_BROWSE);
                 gupnp_service_proxy_add_notify (content_dir,
                                                 "ContainerUpdateIDs",
                                                 G_TYPE_STRING,
@@ -1116,12 +1081,12 @@ gboolean
 get_selected_object (MetadataFunc callback,
                      gpointer     user_data)
 {
-        GUPnPServiceProxy  *content_dir;
-        GtkTreeSelection   *selection;
-        GtkTreeModel       *model;
-        GtkTreeIter         iter;
-        char               *id = NULL;
-        gboolean            ret = FALSE;
+        AVCPMediaServer   *server;
+        GtkTreeSelection  *selection;
+        GtkTreeModel      *model;
+        GtkTreeIter        iter;
+        char              *id = NULL;
+        gboolean           ret = FALSE;
 
         selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (treeview));
         g_assert (selection != NULL);
@@ -1132,11 +1097,11 @@ get_selected_object (MetadataFunc callback,
 
         gtk_tree_model_get (model,
                             &iter,
-                            3, &content_dir,
+                            2, &server,
                             4, &id,
                             -1);
 
-        browse_metadata (g_object_ref (content_dir), id, callback, user_data);
+        browse_metadata (server, id, callback, user_data);
 
         ret = TRUE;
 
@@ -1144,7 +1109,7 @@ get_selected_object (MetadataFunc callback,
                 g_free (id);
         }
 
-        g_object_unref (content_dir);
+        g_object_unref (server);
 
 return_point:
         return ret;
