@@ -86,6 +86,7 @@ get_icon_from_message (SoupMessage    *msg,
         }
 
         gdk_pixbuf_loader_close (loader, NULL);
+        g_object_unref (loader);
 
         return pixbuf;
 }
@@ -294,7 +295,7 @@ init_icons (void)
         }
 
 
-        download_session = soup_session_async_new ();
+        download_session = soup_session_new ();
         g_assert (download_session != NULL);
 
         pending_gets = NULL;
